use warnings;
use strict;

#----------------------------------------------------------------
#  This demonstrates the differences between taking the sprint()
# from the XML::Twig, or the XML::Twig->root() an the impact on
# mixing where the initial data comes from, and how subsequent processing
# of it may change content.
#
# The problem crops up when there is an xml declaration in the file,
# and the file contains a special element, such as a bullet, as found
# in the test_data/heading.xml
#
# the unexpected problem came with the test_data/compound.xml file, that
# has the &lt; as an element. It failed in one of the four cases.
#
#----------------------------------------------------------------
use English qw(-no_match_vars);
use FindBin qw ($Bin);
use Carp;
use Test::More qw(no_plan);
use XML::Twig;

use File::Spec;

use constant TEST_DATA_DIRECTORY => File::Spec->catdir( $Bin, 'test_data', );

#-----------------------------------------------------------------------------

test_twig_v_root_sprint_of_file_data();

exit;

#-----------------------------------------------------------------------------

sub test_twig_v_root_sprint_of_file_data {

    # this will only work IF the 'keep_encoding' is turned off.
    my @fileNames = qw(noHeading.xml heading.xml compound.xml);

    foreach my $fileName (@fileNames) {
        my $file_path = File::Spec->join( TEST_DATA_DIRECTORY, $fileName );

        my $twig_root_from_file = twigRootFromPath($file_path);

        my $xml_from_root = $twig_root_from_file->sprint();

        secondary_processing( $fileName, 'fromRoot', $xml_from_root );

        my $twig_from_file = twigFromPath($file_path);

        my $xml_from_twig = $twig_from_file->sprint();

        secondary_processing( $fileName, 'fromTwig', $xml_from_twig );

    }

    return 1;
}

sub secondary_processing {
    my ( $fileName, $type, $data ) = @_;

    my $twig_root = twigRootFromStringRef( \$data );

    my $got_xml = $twig_root->sprint();

    Test::More::is( $got_xml, $data, "sprinted from root (data from $type) - $fileName" );

    my $twig = twigFromStringRef( \$data );

    my $xml_from_twig = $twig->sprint();

    Test::More::is( $xml_from_twig, $data, "sprinted from twig (data from $type) - $fileName" );

    return;
}

#-----------------------------------------------------------------------------

sub stringRefFromFile {
    # Reads file into a textRef instead of breaking up into lines.
    my ( $path, $mode ) = @_;

    $mode ||= '<:encoding(utf8)';
    local $/;
    open( FILE, $mode , $path )
        or Carp::confess("stringRefFromFile: can't open $path. $OS_ERROR");
    my $fileContents = <FILE>;
    close FILE;

    return \$fileContents;
}

#-----------------------------------------------------------------------------

sub twigRootFromPath {
    my $path    = shift;
    my $xml_ref = stringRefFromFile($path);
    return twigRootFromStringRef($xml_ref);
}

sub twigFromPath {
    my $path    = shift;
    my $xml_ref = stringRefFromFile($path);
    return twigFromStringRef($xml_ref);
}

#-----------------------------------------------------------------------------

sub twigRootFromStringRef {
    my ($xml_ref) = @_;
    my $twig = twigFromStringRef($xml_ref);
    return $twig->root();
}

sub twigFromStringRef {
    my ($xml_ref) = @_;

    my $twig = XML::Twig->new( keep_encoding => 1 );

    $twig->parse( ${$xml_ref} );
    return $twig;
}

#-----------------------------------------------------------------------------
